Attribute VB_Name = "IBUSStuffs"
' BMW I-BUS Analyser.
' Written by Franck Touanen.
' E-Mail : ftouanen@yahoo.fr
'
'
' This source is distributed under GPL licence.
' No commercial usage authorized.


' Module containing various I-BUS Stuffs


' IBUS DEFINITION ----------------------------------------------------

' BMW I-BUS devices ID

' This array will contain Devices text Name
Public IBD_DeviceName(0 To 255) As String

' This string will contain a list of hexa valid ID, comma separated
Public IBD_ValidDevice As String

'0x00 Broadcast 1
Public Const IBD_BRD1 = 0

'0x18 CDC CD-Player
Public Const IBD_CDC = &H18

'0x30. UNKNOW ?????
Public Const IBD_UNKNOW1 = &H30

'0x3F. UNKNOW ?????
Public Const IBD_UNKNOW2 = &H3F

'0x3B NAV Navigation/Videomodule
Public Const IBD_NAV = &H3B

'0x43 MenuScreen
Public Const IBD_MNUSCR = &H43

'0x44. UNKNOW ?????
Public Const IBD_UNKNOW3 = &H44

'0x50 MFL Multi Functional Steering Wheel Buttons
Public Const IBD_MFL = &H50

'0x60 PDC Park Distance Control
Public Const IBD_PDC = &H60

'0x68 RAD Radio
Public Const IBD_RAD = &H68

'0x6A DSP Digital Sound Processor
Public Const IBD_DSP = &H6A

'0x7F. GPS Module ?
Public Const IBD_GPSMOD = &H7F

'0x80 IKE Instrument Control Electronics
Public Const IBD_IKE = &H80

'0xA8. UNKNOW ?????
Public Const IBD_UNKNOW5 = &HA8

'0xBB. TV Module
Public Const IBD_TV = &HBB

'0xBF LCM Light Control Module
Public Const IBD_LCM = &HBF

'0xC0 MID Multi-Information Display Buttons
Public Const IBD_MID = &HC0

'0xC8 TEL Telephone
Public Const IBD_TEL = &HC8

'0xD0 Navigation Location
Public Const IBD_NAVLOC = &HD0

'0xE7 OBC TextBar
Public Const IBD_OBC = &HE7

'0xE8. UNKNOW ?????
Public Const IBD_UNKNOW6 = &HE8

'0xED Lights, Wipers, Seat Memory
Public Const IBD_LIWISEAT = &HED

'0xF0 BMB Board Monitor Buttons
Public Const IBD_BMB = &HF0

'0xFF Broadcast 2
Public Const IBD_BRD2 = &HFF


'-- IBus Messages headers

' Display a text in a specific zone of Nav radio mode

Public Const IB_MSGRAD01 = "A5 62 01"

Public Const IB_MSGRAD02 = "23 62 30"









' Build the Valid Device String

Public Sub InitIbusValues()

Dim i As Integer

' Fill Device Names

IBD_DeviceName(IBD_BRD1) = "Broad. 1  "
IBD_DeviceName(IBD_CDC) = "CD Player "
IBD_DeviceName(IBD_NAV) = "Nav Video "
IBD_DeviceName(IBD_MNUSCR) = "Menuscreen"
IBD_DeviceName(IBD_MFL) = "Steering W"
IBD_DeviceName(IBD_PDC) = "PDC       "
IBD_DeviceName(IBD_RAD) = "Radio     "
IBD_DeviceName(IBD_DSP) = "DSP       "
IBD_DeviceName(IBD_GPSMOD) = "GPS Module"
IBD_DeviceName(IBD_IKE) = "IKE       "
IBD_DeviceName(IBD_TV) = "TV Module "
IBD_DeviceName(IBD_LCM) = "Light Ctrl"
IBD_DeviceName(IBD_MID) = "MID       "
IBD_DeviceName(IBD_TEL) = "TEL       "
IBD_DeviceName(IBD_NAVLOC) = "Nav Locat."
IBD_DeviceName(IBD_OBC) = "OBC       "
IBD_DeviceName(IBD_LIWISEAT) = "Seat Mem.."
IBD_DeviceName(IBD_BMB) = "Board But."
IBD_DeviceName(IBD_BRD2) = "Broad. 2  "

' Unknow at this time

IBD_DeviceName(IBD_UNKNOW1) = "UNKNOW 1  "
IBD_DeviceName(IBD_UNKNOW2) = "UNKNOW 2  "
IBD_DeviceName(IBD_UNKNOW3) = "UNKNOW 3  "
IBD_DeviceName(IBD_UNKNOW5) = "UNKNOW 5  "
IBD_DeviceName(IBD_UNKNOW6) = "UNKNOW 6  "

IBD_ValidDevice = ""

' Initialize array

For i = LBound(IBD_DeviceName) To UBound(IBD_DeviceName)
    
    If IBD_DeviceName(i) <> "" Then
   
       If Len(Hex$(i)) > 1 Then
        IBD_ValidDevice = IBD_ValidDevice & Hex$(i)
       Else
        IBD_ValidDevice = IBD_ValidDevice & "0" & Hex$(i)
       End If
       
          
    End If
        
Next


End Sub

' Analyse IBUS Message From the Hexa Dump box
' Easier for paste and copy to proceed like this rather from a binary
' stream.

' Decode an I-BUS Stream

Public Function DecodeIbusStream(ByRef Stream As Variant) As String

Dim i, Stl, l, Srcid, DestId, cksum As Integer

Dim PkValid, PkUnknow, PkErrors, PkFiltered As Long
Dim packet, ch1, ch2, FilterDest, FilterSrc As String
Dim Filtered, FromTo As Boolean


PkUnknow = 0
PkErrors = 0
PkValid = 0
PkFiltered = 0

'
If frmDumpForm.UnknowIsError.Value <> 1 Then
    UnKnowAreBadPacket = True
Else
    UnKnowAreBadPacket = False
End If


' Check for len. 3 is a minimum for the analyse starts

If Len(Stream) < 3 Then
    DecodeIbusStream = vbCrLf & "** INVALID STREAM LENGTH **" & vbCrLf
    Exit Function
    
End If

Stl = Len(Stream)
For i = 1 To Stl
    
  ' Check for valid ID if this is an error
  ' The "Unknow ID" checkbox is used to consider unknow ID as errors
  ' or just "unknow" IDs.
  
  Srcid = Asc(Mid(Stream, i, 1))
  If (IBD_DeviceName(Srcid) = "" And UnKnowAreBadPacket) Then
        PkErrors = PkErrors + 1
  Else
    If i < Stl Then
       'Yes, get the length
       l = Asc(Mid(Stream, i + 1, 1))
            
       ' Verify length
       ' Not zero length packet
              
       If l = 0 Then
         
                PkErrors = PkErrors + 1
                                               
                        
       ' Check if len coherent
       ' If not, check the Next byte (maybe that len is an ID ?)
       ' If Ok start decoding
       ElseIf (i + l) < Stl Then
                ' Length ok.
                ' Verify Checksum (last byte)
                
                cksum = Asc(Mid(Stream, i + 1 + l, 1))
                
                ' Get The packet but the checksum
                
                packet = Mid(Stream, i, l + 1)
                
                ' Compute regarding the packet
                
                If cksum = compute_cksum(packet, l + 1) Then
                  ' The Packet is a valid One !
                   
                  ' Check the DestId to be sure
                  ' Check for valid ID if this is an error
                    
                  DestId = Asc(Mid(Stream, i + 2, 1))
                  If (IBD_DeviceName(DestId) = "" And UnKnowAreBadPacket) Then
                    PkErrors = PkErrors + 1
                  Else
                    
                    PkValid = PkValid + 1
                    packet = Mid(Stream, i, l + 2)
                    
                    ' Check if Devices are in the filter
                    ' The Checkbox "From/To" Allows to filter
                    ' all messages from/to a device store in Src Dev
                    ' and will generate a COM diagram in the analysis window

                    Filtered = False ' Not filtered by default (rejected)
                    FromTo = True    ' Src by default
                    
                    FilterSrc = Trim(frmDumpForm.DeviceFilterSrc.Text)
                    FilterDest = Trim(frmDumpForm.DeviceFilterDest.Text)
                    ch1 = Hex$(Srcid)
                    ch2 = Hex$(DestId)
                    If Len(ch1) = 1 Then ch1 = "0" & ch1
                    If Len(ch2) = 1 Then ch2 = "0" & ch2
 
                    If FilterSrc <> "" Then
                        
                        If InStr(FilterSrc, ch1) <> 0 Then
                            Filtered = True
                            ' We Are (and stay) SRC
                            
                        End If
                       
                        If frmDumpForm.CheckFromTo.Value = 1 _
                            And Not Filtered Then
                       
                            If InStr(FilterSrc, ch2) <> 0 Then
                                Filtered = True
                                ' We are DEST
                                FromTo = False
                            End If
                            
                        End If
                    Else
                        Filtered = True
                    End If
                                         
                    If FilterDest <> "" Then
                        
                       If frmDumpForm.CheckFromTo.Value <> 1 Then
                          
                           ' Does Dest ID in the Dest Filter ?
                           If InStr(FilterDest, ch2) = 0 Then
                              Filtered = False ' No. Reject packet
                           Else
                              ' Yes. Does the SrcId was in the filter ?
                              Filtered = True And Filtered
                           End If
                                                     
                       End If
                 
                    Else
                        Filtered = True And Filtered
                    End If
                     
                    If Filtered Then
                       PkFiltered = PkFiltered + 1
                       DecodeIbusStream = DecodeIbusStream & DecodeIbusPacket(packet, FromTo) & vbCrLf
                    End If
                                        
                    ' Is it a valid I-BUS Device ID ? (Just for stats)
    
                    If IBD_DeviceName(Srcid) = "" Then
                        PkUnknow = PkUnknow + 1
                    End If

           
                    ' Jump to the next packet (don't forget Next i)
                    
                    i = i + l + 1
                  
                  End If ' Unknow Dest Id
                  
                Else
                    ' It's not valid
                    
                    PkErrors = PkErrors + 1
                
                End If
       Else
        ' Len was not coherent
        PkErrors = PkErrors + 1
       
       End If
                                                     
    
    Else
        ' No, i=stl so this is the last value
        
        PkErrors = PkErrors + 1
        
    End If

  End If ' Unknow ID

Next i

' Display in clear text

DecodeIbusStream = DecodeIbusStream & "-- " & PkValid & " valid packet(s)" & vbCrLf
DecodeIbusStream = DecodeIbusStream & "-- " & PkUnknow & " unknow packet(s)" & vbCrLf
DecodeIbusStream = DecodeIbusStream & "-- " & PkFiltered & " filtered packet(s)" & vbCrLf
DecodeIbusStream = DecodeIbusStream & "-- " & PkErrors & " error(s)" & vbCrLf

DecodeIbusStream = DecodeIbusStream & "-- End Of analysis"




End Function

' Decode an I-BUS packet
' Structure is :
'   SS LL DD DATA.... CK
' Where :
'   SS is Source Device ID
'   LL is len of the packet begining from DD
'   DATA is data packet
'   CK is the Xor Checksum from SS to CK - 1


Function DecodeIbusPacket(ByRef packet As Variant, FromTo As Boolean) As String

    Dim WorkStr, WorkStr2, Data, StrFromTo As String
    Dim Srcid, DestId, l, i, cksum As Integer
 
    Srcid = Asc(Left(packet, 1))
    
    l = Asc(Mid(packet, 2, 1))
    DestId = Asc(Mid(packet, 3, 1))
    Data = Mid(packet, 4, l - 2)
    cksum = Asc(Right(packet, 1))
        
    
    ' Generate Diagram if From/To is activated
    If Not FromTo Then
        StrFromTo = "<---"
    Else
        StrFromTo = ""
    End If
    
   
    ' Start Packet analysis
    
       
    ' Display Source device Id
    
    WorkStr = WorkStr & StrFromTo & "Source Device      : " & IBD_DeviceName(Srcid)
    If IBD_DeviceName(Srcid) = "" Then
        WorkStr = WorkStr & "(Unknow ID =" & Hex$(Srcid) & ")"
    End If
    WorkStr = WorkStr & vbCrLf
    
    ' Display Dest device Id
    WorkStr = WorkStr & StrFromTo & "Destination Device : " & IBD_DeviceName(DestId)
    If IBD_DeviceName(DestId) = "" Then
        WorkStr = WorkStr & "(Unknow ID =" & Hex$(DestId) & ")"
    End If
    WorkStr = WorkStr & vbCrLf
    
    ' Display Len and Checksum
    'WorkStr = WorkStr & StrFromTo & "Data Len (Checksum): " & "(" & Hex$(cksum) & ")" & vbCrLf
    
    ' Display Data
    WorkStr = WorkStr & StrFromTo & "Data  - - - - - - - - - - - -" & vbCrLf
    WorkStr2 = PacketDataAnalyse(GetHexaFromAscii(Data), Srcid, DestId)
    If WorkStr2 = "" Then
       WorkStr = WorkStr & StrFromTo & GetHexaFromAscii(Data) & vbCrLf & StrFromTo

       Data = GetDisplayAbleAscii(Data)
       For i = 1 To Len(Data)
          WorkStr = WorkStr & " " & Mid(Data, i, 1) & " "
       Next i
    Else
       WorkStr = WorkStr & StrFromTo & WorkStr2 & vbCrLf
    End If
    
    ' Display full packet in hexa
    WorkStr = WorkStr & vbCrLf & StrFromTo & "Full Packet - - - - - - - - -" & vbCrLf
    WorkStr = WorkStr & StrFromTo & GetHexaFromAscii(packet) & vbCrLf
    
    WorkStr = WorkStr & StrFromTo & "-----------------------------" & vbCrLf
                        
    DecodeIbusPacket = WorkStr

End Function

' Analyse Data in the IBus packet
' and return Clear text

Function PacketDataAnalyse(HexData As String, Srcid As Variant, DestId As Variant) As String

Dim WorkStr, WorkStr2

WorkStr = ""
Dim Area As Integer

Select Case Srcid

        ' CD Player ---------------------------------------------------------
        Case IBD_CDC
        
            Select Case DestId
            
            ' Broadcast
                
                Case IBD_BRD2
            
         
                    If HexData = "02 00" Then
                    
                        WorkStr = "Poll Resp"
                    End If
            
            
            ' CD Player To Radio
                Case IBD_RAD
       
 
            
                    If Left(HexData, 17) = "39 00 02 00 3F 00" Then
                
                        WorkStr = "Status Not Playing CD " & Right(HexData, 5)
                  
                    ElseIf Left(HexData, 17) = "39 00 09 00 3F 00" Then
                
                        WorkStr = "Status Playing CD " & Right(HexData, 5)
                    
                    ElseIf Left(HexData, 17) = "39 02 09 00 3F 00" Then
            
                        WorkStr = "Start Playing " & Right(HexData, 5)
                    
                    ElseIf Left(HexData, 17) = "39 07 09 00 3F 00" Then
            
                        WorkStr = "End Playing " & Right(HexData, 5)
                    
                    ElseIf Left(HexData, 17) = "39 08 09 00 3F 00" Then
            
                        WorkStr = "Seek for CD" & Right(HexData, 5)
            
                    
                    End If
                
            
            
            End Select
        
        
        ' Radio ------------------------------------------------------------------------------------
        Case IBD_RAD

            Select Case DestId
            
            ' Radio To CD Player
                Case IBD_CDC
            
                    If HexData = "01" Then
                    
                        WorkStr = "Poll CD Player"
            
                    ElseIf HexData = "38 00 00" Then
                    
                        WorkStr = "Request Parameters"
            
                    ElseIf HexData = "38 01 00" Then
                    
                        WorkStr = "Stop Playing"
                        
                    ElseIf HexData = "38 03 00" Then
                    
                        WorkStr = "Request to Play"
            
                    ElseIf HexData = "38 04 00" Then
                    
                        WorkStr = "Scan Backward"
            
                    ElseIf HexData = "38 04 01" Then
                    
                        WorkStr = "Scan Forward"
            
                    ElseIf Left(HexData, 5) = "38 06" Then
                    
                        WorkStr = "Play CD # " & Right(HexData, 2)
            
                    ElseIf HexData = "38 07 00" Then
                    
                        WorkStr = "Scan Mode Off"
            
                    ElseIf HexData = "38 07 01" Then
                    
                        WorkStr = "Scan Mode On"
            
                    ElseIf HexData = "38 08 00" Then
                    
                        WorkStr = "Random Mode Off"
            
                    ElseIf HexData = "38 08 01" Then
                    
                        WorkStr = "Random Mode On"
            
                    ElseIf HexData = "38 0A 00" Then
                    
                        WorkStr = "Next Track"
                    
                    ElseIf HexData = "38 0A 01" Then
                    
                        WorkStr = "Previous Track"
                    
                    End If
            
            ' Radio to Nav
                Case IBD_NAV
            
                    If Left(HexData, 8) = "A5 62 01" Then
                
                        Area = Val("&H" & Mid(HexData, 10, 2))
                        WorkStr = "Display to Video Area # " & Mid(HexData, 10, 2)
                        
                        WorkStr2 = GetAsciiFromHex(Mid(HexData, 13, Len(HexData) - 11))
                        WorkStr = WorkStr & " '" & GetDisplayAbleAscii(WorkStr2) & "'"
                        
                        If Area = 6 Then frmVideoMsg.Area6.Text = WorkStr2
                        If Area = 7 Then frmVideoMsg.Area7.Text = WorkStr2
                        If Area = 5 Then frmVideoMsg.Area5.Text = WorkStr2
                        If Area = 4 Then frmVideoMsg.Area4.Text = WorkStr2
                        If Area = 3 Then frmVideoMsg.Area3.Text = WorkStr2
                        If Area = 2 Then frmVideoMsg.Area2.Text = WorkStr2
                        If Area = 1 Then frmVideoMsg.Area1.Text = WorkStr2
                        
                    ElseIf Left(HexData, 8) = "23 62 30" Then
                            WorkStr = "Display to Video Area #0"
                            WorkStr2 = GetAsciiFromHex(Mid(HexData, 10, Len(HexData) - 9))
                            WorkStr = WorkStr & " '" & GetDisplayAbleAscii(WorkStr2) & "'"
                            frmVideoMsg.Area0.Text = WorkStr2
                    End If
                    'nn <text>
                
            
        
        End Select ' DestId
    
            
    ' Board Monitor Buttons ------------------------------------------------------------------
    Case IBD_BMB
            
                Select Case DestId
                
                    'BMB to Radio
                    Case IBD_RAD
                
                    If HexData = "48 11" Then
                    
                        WorkStr = "Push '1'"
            
                    ElseIf HexData = "48 91" Then
                    
                        WorkStr = "Release '1'"
                        
                    ElseIf HexData = "48 01" Then
                    
                        WorkStr = "Push '2'"
                    
                    ElseIf HexData = "48 81" Then
                    
                        WorkStr = "Release '2'"
                    
                    ElseIf HexData = "48 12" Then
                    
                        WorkStr = "Push '3'"
                    
                    ElseIf HexData = "48 92" Then
                    
                        WorkStr = "Release '3'"
                    
                    ElseIf HexData = "48 02" Then
                    
                        WorkStr = "Push '4'"
                    
                    ElseIf HexData = "48 82" Then
                    
                        WorkStr = "Release '4'"
                        
                    ElseIf HexData = "48 13" Then
                    
                        WorkStr = "Push '5'"
                    
                    ElseIf HexData = "48 93" Then
                    
                        WorkStr = "Release '5'"
                        
                    ElseIf HexData = "48 03" Then
                    
                        WorkStr = "Push '6'"
                    
                    ElseIf HexData = "48 83" Then
                    
                        WorkStr = "Release '6'"
                        
                    ElseIf HexData = "48 32" Then
                    
                        WorkStr = "Push 'TP'"
                    
                    ElseIf HexData = "48 B2" Then
                    
                        WorkStr = "Release 'TP'"
                        
                    ElseIf HexData = "48 22" Then
                    
                        WorkStr = "Push 'RDS'"
                    
                    ElseIf HexData = "48 A2" Then
                    
                        WorkStr = "Release 'RDS'"
                    
                    ElseIf HexData = "48 31" Then
                    
                        WorkStr = "Push 'FM'"
                    
                    ElseIf HexData = "48 B1" Then
                    
                        WorkStr = "Release 'FM'"
                        
                    ElseIf HexData = "48 21" Then
                    
                        WorkStr = "Push 'AM'"
                    
                    ElseIf HexData = "48 A1" Then
                    
                        WorkStr = "Release 'AM'"
                    
                    ElseIf HexData = "48 33" Then
                    
                        WorkStr = "Push 'DOLBY'"
                    
                    ElseIf HexData = "48 B3" Then
                    
                        WorkStr = "Release 'DOLBY'"
                    
                    ElseIf HexData = "48 23" Then
                    
                        WorkStr = "Push 'MODE'"
                    
                    ElseIf HexData = "48 A3" Then
                    
                        WorkStr = "Release 'MODE'"
                        
                    ElseIf HexData = "48 04" Then
                    
                        WorkStr = "Push 'TONE'"
                    
                    ElseIf HexData = "48 84" Then
                    
                        WorkStr = "Release 'TONE'"
                        
                    ElseIf HexData = "48 20" Then
                    
                        WorkStr = "Push 'SELECT'"
                    
                    ElseIf HexData = "48 A0" Then
                    
                        WorkStr = "Release 'SELECT'"
                    
                    ElseIf HexData = "48 10" Then
                    
                        WorkStr = "Push '<'"
                    
                    ElseIf HexData = "48 90" Then
                    
                        WorkStr = "Release '<'"
                    
                    ElseIf HexData = "48 00" Then
                    
                        WorkStr = "Push '>'"
                    
                    ElseIf HexData = "48 A0" Then
                    
                        WorkStr = "Release '>'"
                    
                    ElseIf HexData = "48 30" Then
                    
                        WorkStr = "Push 'Radio Menu'"
                    
                    ElseIf HexData = "48 B0" Then
                    
                        WorkStr = "Release 'Radio Menu'"
                        
                    ElseIf HexData = "48 06" Then
                    
                        WorkStr = "Push 'Radio Roll'"
                    
                    ElseIf HexData = "48 86" Then
                    
                        WorkStr = "Release 'Radio Roll'"
                        
                    ElseIf Left(HexData, 2) = "32" Then
                    
                        If Right(HexData, 1) = "1" Then
                            WorkStr = "'Radio Roll' Rotate Right"
                        ElseIf Right(HexData, 1) = "0" Then
                            WorkStr = "'Radio Roll' Rotate Left"
                        End If
                        
                    End If
                
                
        
                'BMB to Nav
                    Case IBD_NAV
                
                        If HexData = "48 05" Then
                    
                            WorkStr = "Push 'Nav Roll'"
            
                        ElseIf HexData = "48 85" Then
                    
                            WorkStr = "Release 'Nav Roll'"
                            
                        ElseIf Left(HexData, 4) = "49 9" Then
                            
                            WorkStr = "'Nav Roll' Rotate Left"
                        
                        ElseIf Left(HexData, 4) = "49 0" Then
                            
                            WorkStr = "'Nav Roll' Rotate Right"
                        
                        End If
                    
                
                
                End Select
                
    Case IBD_GPSMOD ' GPS Module ----------------------
    
                Select Case DestId
                'GPSMOD to PHONE
                    Case IBD_TEL
                        
                        ' GPS Coordinates + Time
                        If Left(HexData, 2) = "A2" Then
                        
                            WorkStr = "GPS Coordinates" & vbCrLf
                            WorkStr = WorkStr & "Latitude  :"
                            WorkStr = WorkStr & Mid(HexData, 10, 2) & " " & Mid(HexData, 13, 2) & "' " + Mid(HexData, 13, 2) & "' " & Mid(HexData, 16, 2) & "." & Mid(HexData, 19, 2) & vbCrLf
                            WorkStr = WorkStr & "Longitude :"
                            WorkStr = WorkStr & Mid(HexData, 22, 2) & " " & Mid(HexData, 25, 2) & "' " & Mid(HexData, 28, 2) & "' " & Mid(HexData, 31, 2) & "." & Mid(HexData, 34, 2) & vbCrLf
                            WorkStr = WorkStr & "Time      :" & Mid(HexData, 46, 5)
'                        > first the GPS coordinates are sent:
'> 7F 14 C8 A2 00 00
'52 33 19 40 00 13 11 51 70 00 00 00
'> 11 00 00
'> 0B
'> in this case Latitude 52 33' 19.4'' / Longitude 13
'> 11' 51.7''



                        
                        
                        End If
                
               
                
                End Select
            
    
    
    ' xxxx
    Case 999999
                
                Select Case DestId
                
                
                End Select
    


End Select ' SrcId
        

PacketDataAnalyse = WorkStr


End Function

Public Function SendMsgToIbus(mymsg As String) As Boolean
'
'  We must first assert the RTS line to reset
'  the collision detect circuit and enable
'  transmit data.
Dim i, j, Retry As Integer
Dim OkTr As Boolean
Dim Ch As Integer
  

    ' 10 retry
    Retry = 10
    For i = 1 To Retry
        OkTr = False
        
        If frmCommon.MSComm1.OutBufferCount = 0 Then
        AssertRTS True
        ' Send to COM
        ' Is it full ?
        
        
        ''For j = 1 To Len(mymsg)
         '
         '   frmCommon.MSComm1.Output = Mid(mymsg, j, 1)
            'Sleep 3
        'Next j
        
        frmCommon.MSComm1.Output = mymsg
        
        ' Check If we still have the line
                
        
        If frmCommon.MSComm1.CTSHolding Then
                
           ' All is clear !
           ' End of the loop
           frmDumpForm.RchDebug.Text = frmDumpForm.RchDebug.Text & "Transmission done !" & vbCrLf
           OkTr = True
           i = Retry
        
        Else
          frmDumpForm.RchDebug.Text = frmDumpForm.RchDebug.Text & "Bus Busy...Try #" & i & vbCrLf
        End If
    
        ' Deassert RTS
        
        AssertRTS False
        
        End If ' BufferCount
    
    Next i
        
    SendMsgToIbus = OkTr
End Function

' Send a message to a specific area of the nav screen
' When in CD/Radio mode
Sub IBusRadNavMsg(NavArea As Integer, NavText As String)


    Dim packet As String
    
         
    Select Case NavArea
    
    
        Case 0
            If NavText = "" Then NavText = Space(11)
        
        Case 1, 2, 3, 4
            If NavText = "" Then NavText = Space(5)
        
        Case 5
            If NavText = "" Then NavText = Space(7)
        
        Case 6, 7
            If NavText = "" Then NavText = Space(20)
    End Select
    
    
    ' Add Message header
    
    If NavArea <> 0 Then
        NavText = GetAsciiFromHex(IB_MSGRAD01) & Chr(NavArea) & NavText
    Else
        NavText = GetAsciiFromHex(IB_MSGRAD02) & NavText
    End If
        
    
    
   
    ' Add SrcID
    packet = Chr(IBD_RAD)
    
    ' Align Length
    packet = packet & Chr(Len(NavText) + 2)
    
    ' Add Dest Id
    
    packet = packet & Chr(IBD_NAV)
    
    ' Add Data
    
    packet = packet & NavText
    
    ' Add CheckSum
    
    packet = packet & Chr(compute_cksum(packet, Len(packet)))
    
    If frmCommon.MSComm1.PortOpen = False Then
        frmCommon.InitCom MyCOMPort
        frmCommon.MSComm1.PortOpen = True
        
    End If
        
    
    ' Send Data to IBus

    If Not SendMsgToIbus(packet) Then
           frmDumpForm.DumpData packet, "Msg not sent : ", True
        
    End If
    
End Sub

' Display a long message with A scroll Effect
Sub IBusRadNavMsgScroll(Iteration As Integer, NavText)

    
    Dim i, j, l, Stepping, larea As Integer
    
    Dim WorkStr As String
    
          
    
    larea = 20
         
    
    
    
    'Stepping = (l - larea) / larea
    'If Stepping <> Int(Stepping) Then Stepping = Int(Stepping) - 1
    
    NavText = NavText & Space(larea)
    l = Len(NavText)
    
    Stepping = 2
    
    For j = 1 To Iteration
        
        WorkStr = Left(NavText, larea)
        frmVideoMsg.Area7.Text = WorkStr
        Call IBusRadNavMsg(7, WorkStr)
        DoEvents
    
        Sleep (100)
        
        i = Stepping
        Do While i < l
        
           
    '           If I <= larea Then
                  'workstr = Space(larea - I) & Mid(NavText, 1, I)
    '           Else
'           workstr = Mid(NavText, I - larea + 1, larea)
    '           End If
               
               WorkStr = Mid(NavText, i, larea)
               
           frmVideoMsg.Area7.Text = WorkStr
           Call IBusRadNavMsg(7, WorkStr)
           
           DoEvents
           
           i = i + Stepping
           If i > l Then i = l
               
           Sleep (300)
     '          I = I + larea / 2
               
               
        Loop
        WorkStr = Mid(NavText, i - larea + 1, larea)
        frmVideoMsg.Area7.Text = WorkStr
        Call IBusRadNavMsg(7, WorkStr)
        DoEvents
        
    '    Sleep (larea * 300)
    
        frmVideoMsg.Area7.Text = ""
        Call IBusRadNavMsg(7, Space(larea))
        DoEvents
        
    Next j
    
            
                
            
    


End Sub

    
    
    
     ' RchHexa.Text = RchHexa.Text & vbCrLf & "     "



' Pull RTS High or Down
Public Function AssertRTS(High As Boolean) As Boolean

Dim Retry As Integer


Retry = 10

If High Then
    
    frmCommon.MSComm1.RTSEnable = False
    Sleep RTSDelay
    frmCommon.MSComm1.RTSEnable = True
    Sleep RTSDelay
    
    While Not frmCommon.MSComm1.CTSHolding And Retry > 0
        DoEvents
        frmCommon.MSComm1.RTSEnable = False
        Sleep RTSDelay
        
        frmCommon.MSComm1.RTSEnable = True
        Sleep RTSDelay
        Retry = Retry - 1
    
    Wend
    
Else
    
    frmCommon.MSComm1.RTSEnable = False
    Sleep RTSDelay
    While frmCommon.MSComm1.RTSEnable And Retry > 0

        DoEvents
        frmCommon.MSComm1.RTSEnable = True
        Sleep RTSDelay
        
        frmCommon.MSComm1.RTSEnable = False
        Sleep RTSDelay
    Wend

End If

End Function


' Perform a XOR checksum for Mymsg
' Note : the XOR byte must be set to 00 if passed in MyMsg
' The XOR byte is calculated on the full message :
' Sender ID, Len, Dest Id, Data,Cksum set to zero first

Function compute_cksum(ByRef mymsg As Variant, mymsglen As Integer) As Integer

    Dim cksum, k As Integer

    cksum = 0

    For k = 1 To mymsglen

        cksum = cksum Xor Asc(Mid(mymsg, k, 1))
        

    Next k
    
    compute_cksum = cksum
End Function
'


' Transform an HEX string to an Ascii string

Public Function GetAsciiFromHex(ByRef mymsg As Variant)

    Dim i, hexcode As Integer
    Dim WorkStr As Variant
     
    WorkStr = ""
    For i = 1 To Len(mymsg)
        If Mid$(mymsg, i, 1) <> " " Then
            
            hexcode = Val("&H0" & Mid$(mymsg, i, 2))
            i = i + 1
            WorkStr = WorkStr & Chr(hexcode)
                    
        End If
    Next i

    GetAsciiFromHex = WorkStr
End Function

' Get a displayable string by masking specific values by "."

Public Function GetDisplayAbleAscii(ByRef mymsg As Variant) As String
    
    Dim i, Ch As Integer
    Dim WorkStr As String
    
    WorkStr = ""
    
    For i = 1 To Len(mymsg)
        
     Ch = Asc(Mid(mymsg, i, 1))
     If Ch >= 32 And Ch < 127 Then
        WorkStr = WorkStr & Chr(Ch)
     Else
        WorkStr = WorkStr & "."
     End If
    Next i
 
    GetDisplayAbleAscii = WorkStr
    
End Function

' Return an HEXA formatted string from an ascii string
Public Function GetHexaFromAscii(ByRef StrAsc As Variant)

Dim i, Ch As Integer
Dim Hexa, WorkStr As String


For i = 1 To Len(StrAsc)
    
    Ch = Asc(Mid(StrAsc, i, 1))
    Hexa = Hex$(Ch)
    
    If Len(Hexa) = 1 Then
       Hexa = "0" & Hexa
    End If
    
    WorkStr = WorkStr & Hexa & " "
Next i

GetHexaFromAscii = Trim(WorkStr)

End Function

